#!/usr/bin/perl
#
#  generic Whois
#
#  (c) 1998-2005 by Michael Holzt <kju@debian.org> and
#                   Lutz Donnerhacke <Lutz.Donnerhacke@Jena.Thur.de>
#
#  Distribution, usage etc. pp. regulated by the current version of GPL.
#
#
#
# History:
# 2005-04-19  Bodo Eggert
#                   Changed into a perl module, some bugfixes, new option '-q'
# 2005-01-15  kju   Added version output
# 2005-01-13  kju   Added "search" method 'notice' (for giving advice
#                   when no whois/lookup is available)
#                   Added "cgipostcurl" method which uses curl to
#                   fake a referer which is needed for the .nic.ar
#                   (i hate you!)
# 2005-01-06  kju   new routines for searching ipv4 addresses for more
#                   speed improvement
# 2005-01-03  kju   To get rid of the ERX tables which slow down everything,
#                   requery is now ok and will no longer send mails.
#                   (Debian Bug#243148)
#                   Added handling code for .pe (grr) (Debian Bug#264794)
# 2004-03-07  kju   Totally changed the ip address handling code, now uses
#                   CIDR blocks.
# 	                Recognized 'ipas a.b.c.d' as a request to ask team
#		            cymru ip->as mapper whois
# 2004-03-03  kju   Integrated smaller fixes and changes done by lutz.
#                   Removed special code for publicinterestregistry, as
#                   transition is over, and referrers are nolonger used.
#                   Changed handling code for generic whois, can now take
#                   some parameters, removing need for special cases.
# 2003-07-29  kju   fixed publicinterestregistry which is now in transition
#		            phase integrating the whois data
# 2003-02-15  kju   fixed handling of dpkg-old pattern versions
# 2003-01-27  kju   whois.publicinterestregistry.net is now authorative for
#                   .org and needs another special treatment... :-(
#                   plus some minor fixes
# 2003-01-02  f2u   Add "+" for IP address queries to ARIN whois.
#                   Support multiple pattern files.
# 2002-11-01  cord  Small workaround for whois-servers that return ^M.
# 2002-01-17  kju   Reporting if query to upstream whois server failed
#		            (Debian Bug#122535)
# 2000-12-08  kju   Braindead answers from nsiregistry excluded (NSI again!)
# 2000-06-29  lutz  Multiple whois queries (kju)
# 2000-02-29  lutz  Removed obsolete cgibr.
#             lutz  Fixed bug in IP queries.
# 2000-02-08  lutz  Braindead answers from nsiregistry excluded
#                   Workaround for bug@internic removed.
# 1999-12-01  kju   Support for new Internic two-step Whois
# 1999-08-04  lutz  Minor fixed
# 1999-07-01  lutz  Allowed spaces, commas and colons
# 1999-04-12  lutz  Hotfix for problems with LWP::Simple
# 1998-12-21  lutz  solving problems with ripe handles.
# 1998-12-18  lutz  Mirrors possible
#             kju   wwwsgrep with LWP::Simple
#             lutz  multiple options bug fixed
#             kju   whois access made with LWP::Simple
#             lutz  Cosmetic changes,
#                   Configurable,
#                   Dies without line numbers,
#                   Direct server access
#             kju   External Configuration File,
#                   Major rewrite
# 1998-12-17  lutz  Published due to kju
#

package Net::Gwhois;

use strict;
use warnings;
use LWP::Simple;
use Socket;
use Carp;


BEGIN {
	use Exporter   ();
	our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

	# set the version for version checking
	$VERSION = 20050115;

	@ISA         = qw(Exporter);
	@EXPORT      = qw(
		&gwhois
		);
	%EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],

	@EXPORT_OK   = qw(version_details); #qw(&func3);
}
our @EXPORT_OK;
our $VERSION;

my $confdir_default='/etc/gwhois';
my $patternfilename = 'pattern';
my $patternfilere = qr/^pattern/;

sub version_details(%){
	my %opt=@_;
	my $user_home;
	if(defined $ENV{'HOME'}){$user_home=$ENV{'HOME'}}
	if(defined $opt{home}){$user_home=$opt{home}}
	if(!defined $user_home){$user_home="/var/home/whois"}
	if(! defined $opt{confdir}){$opt{confdir}=$confdir_default}
	local *PATTERN;


	if ( -f "/etc/default/gwhois" ) {
		require "/etc/default/gwhois";
	}
	my $r = "gwhois - generic whois\n\n".
	        "program version:  $VERSION\n".
	        "pattern tables:   ";
	foreach my $patternfile (&getpatternfiles($opt{confdir})) {
		$patternfile = "$opt{confdir}/$patternfile";
		if ( open(PATTERN,"<$patternfile") ) {
		  my $line = <PATTERN>;
		  close(PATTERN);

		  my ($version) = $line =~ /#:\s+version\s+(\S+)/;
		  $version = 'unknown' if ( $version eq '' );
		  $r .= "$version\t($patternfile)\n                  ";
		}
	}
	$r .= "\n";
	return $r;
}

# $result = &whoisaccess($host,$port,$query,%opt)
# ($result, $success_status) = &whoisaccess($host,$port,$query,%opt)
# $opt{useLWP} => 1;
sub whoisaccess($$$%) {
	my ($host,$port,$query,%opt) = @_;
	my $result;
	my $status = 1;
	my $useLWP = (defined $opt{useLWP})?$opt{useLWP}:1;

	$query =~ s/ /%20/g;
	if (!defined( $result = $useLWP ? LWP::Simple::get("gopher://$host:$port/0$query")
		              : qx{lynx -source gopher://$host:$port/0$query} ) )
	{
		$result = 'Query to whois server failed.';
		$status = 0;
	}

	$result =~ s/\x0d\x0a/\n/gs;
	if (wantarray) {return ($result, $status)}
	return $result;
}

# $result = &inicwhoisaccess($host,$port,$query,%opt)
# ($result, $success_status) = &inicwhoisaccess($host,$port,$query,%opt)
# $opt{useLWP} => 1;
# $opt{verbose} => undef;

# $result = &inicwhoisaccess($host,$port,$query,%opt)
sub inicwhoisaccess($$$%) {
	my ($host,$port,$query,%opt) = @_;
	my ($queryresult, $result, $result2,$status);

	($queryresult,$status) = whoisaccess($host,$port,"=$query",%opt);

	# Result von NSI-Registry auf relevanten Part absuchen
	if ( $status && $queryresult =~ /Name:\s+$query\s/mi ) {
		$result = "-- From: $host:$port\n\n";
		($host) = $queryresult =~
			/Name:\s+$query\s.*?Whois Server:\s+(.*?)\s/si;
		my ($relresult) = $queryresult =~
			/[\r\n]([^\r\n]+\S+\sName:\s+$query\s.*?URL[^\r\n]+)[\r\n]/si;

		$result .= "$relresult\n\n-- End --\n\n";

		$port = 43;
		if($opt{verbose}) {
			print STDERR "Querying $host:$port with whois.\n\n";
		}

		$result .= "-- From: $host:$port\n\n";
		my $r;
		($r,$status) = whoisaccess($host,$port,$query,%opt);
		$result .= $r;
	} else {
		$result = "-- From: $host:$port\n\n$queryresult-- End --\n";
	}

	if (wantarray) {return ($result, $status)}
	return $result;
}

# $result = &wwwsgrep($url,$match,%opt)
# $opt{useLWP} => 1
sub wwwsgrep($$%) {
	my ($url,$match,%opt) = @_;
	my ($res, $line);
	my $useLWP = (defined $opt{useLWP})?$opt{useLWP}:1;

	$line = $useLWP ? LWP::Simple::get($url)
		            : qx(lynx -source $url);
	$line =~ s/\n/ /g;
	if ($line =~ $match) {
		($res) = $line =~ /$match/s;
	}
	return $res;
}

# ($host, $additional) = &methodpatternregex($query,$host,$additional,$queryline);
sub methodpatternregex($$$$) {
	my ($query,$host,$additional,$line) = @_;

	my ($namewotld,$tld) = $query =~ /^(.*)\.([^.]*)$/;
	my ($p1,$p2) = $query =~ $line;
	my ($ucq) = uc($query);

	$host       =~ s/~query~/$query/;
	$host       =~ s/~ucquery~/$ucq/;
	$host       =~ s/~namewotld~/$namewotld/;
	$host       =~ s/~tld~/$tld/;
	$host       =~ s/~1~/$p1/;
	$host       =~ s/~2~/$p2/;

	if (defined $additional) {
		$additional =~ s/~query~/$query/;
		$additional =~ s/~ucquery~/$ucq/;
		$additional =~ s/~namewotld~/$namewotld/;
		$additional =~ s/~tld~/$tld/;
		$additional =~ s/~1~/$p1/;
		$additional =~ s/~2~/$p2/;
	}

	return ($host,$additional);
}

# @patternfiles = &getpatternfiles($confdir)
sub getpatternfiles($) {
	my $confdir=$_[0];
	my (@files);

	# Find available pattern files
	opendir(DIR, "$confdir");
	@files = sort(readdir(DIR));
	closedir(DIR);

	# Move "pattern" (main file) to the end, filter non-pattern files (and old -erx ones).
	@files = grep { ($_ =~ /$patternfilere/)
			&& ($_ ne $patternfilename) && ($_ !~ /dpkg-old/) && ($_ !~ /-erx/) } @files;
	push @files, $patternfilename;

	return (@files);
}

# ($method, $host, $additional) = &getmethodother($query,%opt);
# $opt{confdir}
sub getmethodother($%) {
	my ($query,%opt) = @_;
	my $found=0;
	my ($line,$cline,$method,$host,$additional,@files);

	my %mirror;
	if (defined $opt{mirror}) {
		if(! ref $opt{mirror} eq "HASH") {
			confess("argument mirror is not a hash reference");
		}
		%mirror=%{$opt{mirror}};
	}

	# Process file until we found a match
	foreach my $patternfile (&getpatternfiles($opt{confdir})) {
		$patternfile = "$opt{confdir}/$patternfile";

		open(PATTERN,"<$patternfile") || confess "Cannot open $patternfile. STOP.\n";

		while ( defined($line = <PATTERN>) && (!$found) ) {
			chomp $line;

			if ( $line =~ /^#/ ) {                       # comment
			} elsif ( ($cline) = $line =~ /^:(.*)$/ ) {  # method declaration
			  ($method,$host,$additional) = split(/\|/,$cline,3);
			} elsif ( $line ne '' && $query =~ $line ) {
			  $found = 1;
			  ($host,$additional) = &methodpatternregex($query,$host,$additional,$line);
			}
		}

		}
		if (!$found) {
		return ('','','')
		}
	$host = $mirror{$method.$host} if defined $mirror{$method.$host};
	return ($method,$host,$additional);
}

# %v4pattern = &getpatternv4(%opt)
# $opt{confdir}
sub getpatternv4(%) {
	my %opt=@_;
	my (%pattern);
	my ($method,$host,$additional,$cline,$line,$rehost,$readditional);

	foreach my $patternfile (&getpatternfiles($opt{confdir})) {
		$patternfile = "$opt{confdir}/$patternfile";
		open(PATTERN,"<$patternfile") || confess "Cannot open $patternfile. STOP.\n";
		my $found=0;

		while ( defined($line = <PATTERN>) && (!$found) ) {
			chomp $line;

			if ( $line =~ /^#/ ) {                       # comment
			} elsif ( ($cline) = $line =~ /^:(.*)$/ ) {  # method declaration
				($method,$host,$additional) = split(/\|/,$cline,3);
			} elsif ( $line =~ /^=/ ) {
			  ($rehost,$readditional) = &methodpatternregex('',$host,$additional,$line);
			  $pattern{$line}{'method'} = $method;
			  $pattern{$line}{'host'}   = $rehost;
			  $pattern{$line}{'add'}    = $readditional;
			}
		}
	}
	return (%pattern);
}

# ($method, $host, $additional) = &getmethodv4($$$$query,%opt);
# $opt{confdir}
sub getmethodv4($$$$%) {
	my ($ipa, $ipb, $ipc, $ipd, %opt) = @_;
	my ($ip, $bits, $netmask, $method, $host, $additional, %pattern);

	$ip      = $ipa<<24|$ipb<<16|$ipc<<8|$ipd;
	$netmask = 256**4-1;
	%pattern = &getpatternv4(%opt);

	for ($bits=32; $bits>=0 && !$method; $bits--) {
		$ip        = $ip & $netmask;
		$netmask <<= 1;
		my $cidr;

		if ( $bits > 24 ) {
			$cidr = sprintf("%d.%d.%d.%d/$bits", $ip>>24,($ip>>16)&255,($ip>>8)&255,$ip&255);
		} elsif ( $bits > 16 ) {
			$cidr = sprintf("%d.%d.%d/$bits", $ip>>24,($ip>>16)&255,($ip>>8)&255);
		} elsif ( $bits > 8 ) {
			$cidr = sprintf("%d.%d/$bits", $ip>>24,($ip>>16)&255);
		} else {
			$cidr = sprintf("%d/$bits", $ip>>24);
		}

		$method     = $pattern{"=$cidr"}{'method'};
		$host       = $pattern{"=$cidr"}{'host'};
		$additional = $pattern{"=$cidr"}{'add'};
	}

	return ($method,$host,$additional);
}

# $result=&doquery($query,$method,$host,$additional,%opt);
# $opt{confdir}
# $opt{verbose}
sub doquery($$$$%) {
	my ($query,$method,$host,$additional,%opt) = @_;
	my $result;
	my $status;
	
	if ($method eq 'wwwgreplv') {
		my ($a,$b) = split(/#/,$host);
		my $tmp;

		if ($b) {
			$tmp = uc($b);
			$tmp = 'Oth' if $tmp !~ /[A-Z]/;
		} else {
			$tmp = uc($a);
		}
		$method = "wwwsgrep";
		$host = "http://www.nic.lv/DNS/list$tmp.htm";
	}

	if ($method eq 'wwwsgrep') {
		my ($hostname) = $host =~ /http:\/\/([^\/]*)\//;
		my $res;

		if($opt{verbose}) {
			print STDERR "Querying $hostname with http.\n\n";
		}

		$res = &wwwsgrep($host,$additional);

		if ($res ne '') {
			$result = "Match found:\n$res";
			$status=1;
		} else {
			$result = "No match found. This probably means that this domain does not exist.";
			$status=0;
		}
	} elsif ($method eq 'wwwpe') {
		# why does every shitty nic need to invent its own standard?
		my ($hostname) = $host =~ /http:\/\/([^\/]*)\//;
		my $res;

		if($opt{verbose}) {
			print STDERR "Querying $hostname with http.\n";
		}

		$res = &wwwsgrep($host,$additional);

		if ($res ne '') {
			$result = "Match found. Now querying for the domain data.\n\n";
			$result .= &doquery($query,'cgi','http://www.nic.pe/' . $res, '', %opt);
			$status=1;
		} else {
			$result = "No match found. This probably means that this domain does not exist.";
			$status=0;
		}
	} elsif ($method eq 'wwwbm') {
		# why does every shitty nic need to invent its own standard?
		my ($hostname) = $host =~ /http:\/\/([^\/]*)\//;
		my $res;

		if($opt{verbose}) {
			print STDERR "Querying $hostname with http.\n";
		}

		# Get session URL
		$res = &wwwsgrep($host,$additional);

		# Get lookup
		($result, $status) = &doquery($query,'cgipost',"http://$hostname$res",
			"ADOM++++++=$query&_PROCESS=BMWHO+&_FUNCTION=BMWHO2+",
		);

	} elsif ($method eq 'wwwbz') {
		# why does every shitty nic need to invent its own standard?
		my $hexstring;
		$hexstring = $query;
		$hexstring =~ s/./sprintf("%02x",ord("$&"))/eg;

		($result, $status) = &doquery($query,'cgi',$host.$hexstring,'');
	} elsif ($method =~ /^whois(|jp|arin)$/) {
		my ($port,$noipprefix,$ipprefix,$trailer,$strip,$parameter,$outquery);
		
		$port       = 43;
		$noipprefix = '';
		$ipprefix   = '';
		$trailer    = '';
		$strip      = '';

		if ($additional) {
			foreach $parameter (split('\|', $additional)) {
				$port       = $1 if ( $parameter =~ /port=(\d+)/ );
				$noipprefix = $1 if ( $parameter =~ /noipprefix=(.*)/ );
				$ipprefix   = $1 if ( $parameter =~ /ipprefix=(.*)/ );
				$trailer    = $1 if ( $parameter =~ /trailer=(.*)/ );
				$strip      = $1 if ( $parameter =~ /strip=(.*)/ );
		}	}

		if($opt{verbose}) {
			print STDERR "Querying $host:$port with whois.\n";
		}

		if ( $query =~ /^\d+\.\d+\.\d+\.\d+$/ ) {
			$outquery = $ipprefix . $query . $trailer . "\n";
		} else {
			$outquery = $noipprefix . $query . $trailer . "\n";
		}

		$outquery =~ s/$strip//g if ( $strip ne '' );

		($result, $status) = &whoisaccess($host,$port,$outquery,%opt);

		if ( $status ) {
			if( $result =~ /ReferralServer: whois:\/\/(.*):(\d+)/mi ) {
				if($opt{verbose}) {
					print STDERR "Querying $1:$2 with whois.\n";
				}
				($result, $status) = &whoisaccess($1,$2,$query,%opt);
			} elsif ( $result =~ /ReferralServer: whois:\/\/(.*)/mi ) {
				if($opt{verbose}) {
					print STDERR "Querying $1:43 with whois.\n";
				}
				$result = &whoisaccess($1,43,$query,%opt);
			}
			$result.= "\n";
		}
	} elsif ($method eq 'inicwhois' ) {
		my $port = $additional || 43;
		if($opt{verbose}) {
			print STDERR "Querying $host:$port with whois.\n";
		}
		$query .= "\n";
		($result, $status) = &inicwhoisaccess($host,$port,$query,%opt);
	} elsif ($method eq 'cgi') {
		my ($hostname) = $host =~ /http:\/\/([^\/]*)\//;
		if($opt{verbose}) {
			print STDERR "Querying $hostname with cgi.\n\n";
		}
		#!!
		$result="$host\n";
		$result .= `lynx -dump "$host"`;
		$status=!$?;
	} elsif ($method eq 'cgipost') {
		my ($hostname) = $host =~ /http:\/\/([^\/]*)\//;
		if($opt{verbose}) {
			print STDERR "Querying $hostname with cgi.\n\n";
		}
		#!!
		if($opt{verbose}) {
			print STDERR "echo -e '$additional\n---' | lynx -dump -post_data '$host'\n";
		}
		$result = `echo -e "$additional\n---" | lynx -dump -post_data "$host"`;
		$status=!$?;
	} elsif ($method eq 'cgipostcurl') {
		my ($hostname) = $host =~ /https{0,1}:\/\/([^\/]*)\//;
		if($opt{verbose}) {
			print STDERR "Querying $hostname with cgi.\n\n";
		}
		#    print "$additional\n"; #!!
		#    print "curl --stderr /dev/null -e $host --data '$additional' $host | lynx -dump -stdin\n";
		$result = `curl --stderr /dev/null -e "$host" --data "$additional" "$host" | lynx -dump -stdin`;
		$status=!$?;
	} elsif ($method eq 'cgihttps') {
		my ($hostname) = $host =~ /https:\/\/([^\/]*)\//;
		if($opt{verbose}) {
			print STDERR "Querying $hostname with cgi.\n\n";
		}
		#    print "$additional\n"; #!!
		#    print "curl --stderr /dev/null $host | lynx -dump -stdin\n";
		$result = `curl --insecure --stderr /dev/null "$host" | lynx -dump -stdin`;
		$status=!$?;
	} if ($method eq 'notice') {
		$result = "\n\nNo lookup service available for your query '$query'.\ngwhois remarks: " . $additional . "\n\n";
		$status=0;
	}

	if($status) {

		if ($host =~ /arin/) {
			if ($result =~ /Maintainer: RIPE/) {
				($result, $status) = &doquery($query,'whois','whois.ripe.net','');
			} elsif ($result =~ /Maintainer: AP/) {
				($result, $status) = &doquery($query,'whois','whois.apnic.net','');
			}
		} elsif ($host =~ /apnic/) {
			if ($result =~ /netname: AUNIC-AU/) {
				($result, $status) = &doquery($query,'whois','whois.aunic.net','');
			} elsif ($result =~ /netname: JPNIC-JP/) {
				($result, $status) = &doquery($query,'whoisjp','whois.nic.ad.jp','');
			}
		}

		if ($host =~ /ripe/ && $result =~ /remarks:\s+whois -h (\S+)/) {
			($result, $status) = &doquery($query,'whois',$1,'');
		}

		if (($host =~ /internic/) && ($result =~ /No match for/) &&
			($query !~ /\.(arpa|com|edu|net|org)$/) ) {
			my $result1;
			($result1, $status) = &doquery($query,'whois','whois.ripe.net','');
			if ($result1 !~ /No entries found/) {
				$result = $result1;
			} else {$status = 0}
		}
	}

	if (wantarray) {return ($result, $status)}
	return $result;
}


# ($result, $status) = gwhois($query, %opt)
# home    => $HOME || "/var/home/whois"
# confdir => "/etc/gwhois"
# host    => undef
# additional=>document_me
# mirror  => {}
# useLWP  => 1
# verbose => undef
sub gwhois($%){
	my ($query, %opt)=@_;
	my $user_home;
	local $| = 1;
	if(defined $ENV{'HOME'}){$user_home=$ENV{'HOME'}}
	if(defined $opt{home}){$user_home=$opt{home}}
	if(!defined $user_home){$user_home="/var/home/whois"}
	if(! defined $opt{confdir}){$opt{confdir}=$confdir_default}
	my $r;
	my $status=0;

	my $patternfilename = 'pattern';
	my $patternfilere = qr/^pattern/;
	my $sendmail = 1;
	if ( -f "/etc/default/gwhois" ) {
		require "/etc/default/gwhois";
	}

	$query =~ s/^\s+//;
	$query =~ s/\s+$//;
	$query =~ y/[\xA0-\xFF]a-zA-Z0-9:.,+_ -//cd;
	$query =~ s/\.$//;

	my ($method,$host,$additional);
	$r = "Process query: '$query'\n";
	if ($opt{host}) {
		($method,$additional,$host)=('whois','',$opt{host});
		if(defined $opt{additional}) {$additional=$opt{additional}}
		elsif($opt{host} =~ /^whois\.nic\.ad\.jp(?::|$)/){$additional='trailer=/e'}
		if(defined $opt{method}){$method=$opt{method}}
	} else {
		if ($query =~ /(?:\s|^)(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})(?:\s|$)/) { # ipv4
			my ($a, $b, $c, $d)=($1,$2,$3,$4);
			$r .= "Query recognized as IPv4.\n";
			($method,$host,$additional) = &getmethodv4($a,$b,$c,$d,%opt);
		} elsif ( lc($query) !~ /[^0-9a-f:]/ ) { # ipv6
			# check and correct v6 address
			if ( $query =~ /[0-9a-f]{5}/ || $query =~ /:::/ ) {
				$r .= "$query is an invalid IPv6-Address.\n";
				goto out_err;
			}
			$query =~ s/:?$/::/ if ( $query !~ /(.*:){7}/ && $query !~ /::/ );
			$r .= "Query recognized as IPv6.\n";
			($method,$host,$additional) = &getmethodother($query,%opt);
		} else {
			($method,$host,$additional) = &getmethodother($query,%opt);
		}
	}

	if (!$method) {
		$r.= "I don't know where to query that.\n";
		goto out_err;
	}

	my $result;
	($result, $status) = &doquery($query,$method,$host,$additional,%opt);
	if(defined $result) {$r .= $result."\n\n"}

	if($status) {
		$r .= "\n-- \n  To resolve one of the above handles: ";
		if ($method =~ /whois/) {
			$r .= "whois -h $host";
			$r .= ":$1" if ( $additional && $additional =~ /port=(\d+)/ );
			$r .= " HANDLE\n";
		} elsif ($method eq "cgipost") {
			$r .= "\n     POST $host\n";
			$r .= "     $additional\n";
		} elsif ($method eq "cgi") {
			$r .= "\n     $host\n";
		} elsif ($method eq "wwwgrep") {
			$r .= "\n     hmm. not sure.\n";
		}
	}

	$r .= "  OTOH offical handles should be recognised directly.\n";
	$r .= "  Please report errors or misfits via the debian bug tracking system.\n";

#
	goto out;
	out_err:
	$status=0;
	out:
	if (wantarray) {return ($r, $status)}
	return $result;
}

sub main(@){
	my %opt=('verbose' => 1);
	my %mirror=();
	while(@ARGV && $ARGV[0] =~ /^-/) {
		if($ARGV[0] eq '--help'
		|| $ARGV[0] eq '-?') {
			print "gwhois - generic whois\n",
			"Version $::VERSION\n\n",
			"Usage: gwhois {options} [query]\n",
			" Try find information about the query (might be multiple words).\n",
			" If no query is given, use the first line from stdin\n\n",
			" Options:\n",
			"   -C dir      setting an alternate configuration directory\n",
			"               default $confdir_default\n",
			"   -h host     selecting a fixed whois server for this query\n",
			"   -m method:host host   Defining a mirror for a given method and host.\n",
			"   -L          use lynx -source instead of LWP::Simple\n",
			"   -v          output version of pattern table(s)\n",
			"   -q          be quiet\n",
			"   -?, --help  printing this text\n\n";
			exit;
		} elsif($ARGV[0] eq '-C') {
			shift(@ARGV);
			$opt{confdir}=shift(@ARGV);
		} elsif($ARGV[0] eq '-h') {
			shift(@ARGV);
			$opt{host} = shift(@ARGV);
		} elsif($ARGV[0] eq '-L') {
			shift(@ARGV);
			$opt{useLWP} = 0;
		} elsif($ARGV[0] eq '-m') {
			shift(@ARGV);
			$_ = shift(@ARGV);
			s/://;
			$mirror{$_}=shift(@ARGV);
		} elsif($ARGV[0] eq '-v') {
			print version_details(%opt);
			exit 0;
		} elsif($ARGV[0] eq '-q') {
			shift(@ARGV);
			delete($opt{verbose});
		} else {croak("Unknown option: $ARGV[0]")}
	}
	if(%mirror) { $opt{mirror}=\%mirror;}
	my $status=1;
	for my $q (@ARGV){
		my ($r, $s) = gwhois($q,%opt);
		if (!$s) {$status=0}
		print $r;
	}
	exit !$status;
}
